window.CBS = Ember.Application.create();

CBS.ERROR_MESSAGE_MAP = {
  "The user credentials were incorrect.": "Invalid username/password!",
  "The request is missing a required parameter, includes an invalid parameter value, includes a parameter more than once, or is otherwise malformed. Check the \"username\" parameter.": "Please enter the username.",
  "The request is missing a required parameter, includes an invalid parameter value, includes a parameter more than once, or is otherwise malformed. Check the \"password\" parameter.": "Please enter the password."
}

CBS.env = (window.location.href.match(/cbs\-app/) !== null ? 'development' : 'production');
CBS.config = {
  apiEndpoint: (CBS.env == 'development' ? 'http://callbackstaffing.dev/api' : 'https://www.callbackstaffing.com/api')
}

CBS.utilities = {
  snackbarTimeout: null,
  showSnackbar: function(message, delay) {
    if (typeof delay === 'undefined') {
      delay = 4000;
    }

    if (this.snackbarTimeout) {
      clearTimeout(this.snackbarTimeout);
    }

    $('#snackbar span').html(message);
    $('#snackbar').addClass('active');
    if (delay) {
      this.snackbarTimeout = setTimeout(function(){
        CBS.utilities.hideSnackbar();
      }, delay);
    }
  },
  hideSnackbar: function() {
    $('#snackbar').removeClass('active');
  }
}

function api(route) {
  return CBS.config.apiEndpoint + route;
}

CBS.SlideInView = {
  didInsertElement: function(){
    //called on creation
    this.$()
      .addClass('animated-content');

    Ember.run.next(this, function(){
      this.$()
        .addClass('active-content');
    });
  }
};

CBS.ApplicationRoute = Ember.Route.extend(Ember.SimpleAuth.ApplicationRouteMixin, {
  actions: {
    sessionAuthenticationFailed: function(error) {
      var errorMessage = (typeof CBS.ERROR_MESSAGE_MAP[error.error_description] !== 'undefined'
        ? CBS.ERROR_MESSAGE_MAP[error.error_description]
        : error.error_description
      );
      this.controllerFor('login').set('loginErrorMessage', errorMessage);
      $('#login-form button').button('reset');
    },
    sessionAuthenticationSucceeded: function() {
      app.registerPushNotificationServices();

      return this._super();
    },
    sessionInvalidationSucceeded: function() {
      window.localStorage.removeItem('current-user');
      window.localStorage.removeItem('registration-id');
      window.localStorage.removeItem('old-registration-id');
      window.localStorage.removeItem('user-schedule');
      window.localStorage.removeItem('users');
      window.localStorage.removeItem('groups');
      window.localStorage.removeItem('filters');
      window.localStorage.removeItem('titles');

      this.transitionTo('login');
    },
    backButton: function(){
      history.go(-1);
    }
  }
});

Ember.Application.initializer({
  name: 'authentication',
  initialize: function(container, application) {
    container.register('authenticator:cbs', CBS.CustomAuthenticator);
    Ember.SimpleAuth.Authenticators.OAuth2.reopen({
      serverTokenEndpoint: api('/oauth/access_token'),
  		makeRequest: function(data) {
  			data.client_id = 'yeh4dYMbT6';
  			data.client_secret = 'iQu7eB5nU50E5N7162m5JO8dAJ0RXfDn';
  			if (!Ember.SimpleAuth.Utils.isSecureUrl(this.serverTokenEndpoint)) {
  			  	Ember.Logger.warn('Credentials are transmitted via an insecure connection - use HTTPS to keep them secure.');
  			}
  			return Ember.$.ajax({
  				url:         this.serverTokenEndpoint,
  				type:        'POST',
  				data:        data,
  				dataType:    'json',
  				contentType: 'application/x-www-form-urlencoded'
  			});
  		}
    });

    Ember.SimpleAuth.setup(container, application, {
    	crossOriginWhitelist: ['https://www.callbackstaffing.com', 'http://callbackstaffing.dev/api'],
      routeAfterAuthentication: 'index',
      authorizerFactory: 'authorizer:oauth2-bearer',
      authenticationRoute: 'login'
    });
  }
});

CBS.LoginController = Ember.Controller.extend(Ember.SimpleAuth.LoginControllerMixin, {
  authenticatorFactory: 'authenticator:cbs',
  loginErrorMessage: null
});

CBS.CustomAuthenticator = Ember.SimpleAuth.Authenticators.OAuth2.extend({
  authenticate: function(credentials) {
    this.set('loginErrorMessage', null);
    $('#login-form button').button('loading');
    return this._super(credentials);
  },
  restore: function(data) {
    return this._super(data).then(function(r){
      app.registerPushNotificationServices();
      return r;
    });
  }
});

CBS.ApplicationController = Ember.Controller.extend({
  currentUser: null,
  pageTitle: '',
  userName: '',
  actions: {
    toggleMenu: function() {
      $('#main-menu').trigger('open');
    }
  }
});

Ember.View.reopen({
  didInsertElement : function(){
    this._super();
    Ember.run.scheduleOnce('afterRender', this, this.afterRenderEvent);
  },
  afterRenderEvent: function(){
  }
});

CBS.IndexController = Ember.Controller.extend({
  needs: ['callbacks', 'application'],
  schedule: [],
  hasNextShift: function() {
    return this.get('schedule').length;
  }.property('schedule'),
  getNextShift: function() {
    return this.get('schedule')[0];
  }.property('schedule')
});

CBS.IndexView = Ember.View.extend({
  afterRenderEvent: function(){
    Ember.$('.callback-count-badge').hide();
    Ember.$('.tiny-loader').show();
    Ember.run.once(this.controller.get('controllers.callbacks'), 'send', 'refresh', function(){
      Ember.$('.tiny-loader').fadeOut(100, function(){
        Ember.$('.callback-count-badge').fadeIn(100);
      });
    });

    Ember.$(document).off('click', 'a.external-link').on('click', 'a.external-link', function(e){
      e.preventDefault();
      window.open($(this).attr('href'), '_blank', 'location=yes,closebuttoncaption=Done');
    });
  },
  didInsertElement: function(){
    //called on creation
    this.$()
      .addClass('animated-content-left');

    Ember.run.next(this, function(){
      this.$()
        .addClass('active-content');
    });

    this._super();
  }
});

CBS.CallbacksController = Ember.Controller.extend({
  actions: {
    clear: function() {
      this.set('callbacks', []);
    },
    refresh: function(callbackFunction) {
      var self = this;
      $('.refresh-button').button('loading');
      CBS.Callback.all(true).then(function(callbacks){
        self.set('callbacks', callbacks);
        $('.refresh-button').button('reset');
        if ($.isFunction(callbackFunction)) {
          callbackFunction(self);
        }
      });
    }
  },
  callbacks: [],
  count: function(){
    return this.get('callbacks').length;
  }.property('callbacks')
});

CBS.CallbackController = Ember.Controller.extend({
  actions: {
    deny: function() {
      preloader();
      var callbackRecord = this.get('model');
      var self = this;
      callbackRecord.deny(function(response){
        removePreloader();
        if (response && response.error) {
          var errorview = CBS.ErrorModalView.create({
            title: 'Error',
            content: response.message
          });

          errorview.append();
        }
        else {
          var errorview = CBS.CallbackResponseSuccessModalView.create({
            controller: self,
            title: 'Success',
            content: response.message
          });

          errorview.append();
        }
      });
    },
    accept: function(part) {
      part = part || 'full';
      preloader();
      var callbackRecord = this.get('model');
      var self = this;
      callbackRecord.accept(function(response){
        removePreloader();
        if (response && response.error) {
          var errorview = CBS.ErrorModalView.create({
            title: 'Error',
            content: response.message
          });

          errorview.append();
        }
        else {
          var errorview = CBS.CallbackResponseSuccessModalView.create({
            controller: self,
            title: 'Success',
            content: response.message
          });

          errorview.append();
        }
      }, part);
    },
    acceptWorkSite: function(workSiteId) {
      preloader();
      var callbackRecord = this.get('model');
      var self = this;
      callbackRecord.acceptWorkSite(function(response){
        removePreloader();
        if (response && response.error) {
          var errorview = CBS.ErrorModalView.create({
            title: 'Error',
            content: response.message
          });

          errorview.append();
        }
        else {
          var errorview = CBS.CallbackResponseSuccessModalView.create({
            controller: self,
            title: 'Success',
            content: response.message
          });

          errorview.append();
        }
      }, workSiteId);
    }
  }
});

CBS.NotificationsController = Ember.Controller.extend({
  actions: {
    clear: function() {
      this.set('notifications', []);
    },
    refresh: function(callbackFunction) {
      var self = this;
      self.transitionToRoute('notifications', 1);
    },
    prev: function(){
      this.transitionToRoute('notifications', this.get('currentPage') - 1);
    },
    next: function(){
      this.transitionToRoute('notifications', this.get('currentPage') + 1);
    }
  },
  needs: ['application'],
  currentUser: Ember.computed.alias('controllers.application.currentUser'),
  notifications: [],
  currentUser: null,
  count: function(){
    return this.get('notifications').length;
  }.property('notifications'),
  hasPreviousPage: function(){
    return this.get('currentPage') > 1;
  }.property('currentPage', 'total'),
  hasNextPage: function(){
    return Math.ceil(this.get('total') / CBS.UserNotification.itemsPerPage) > this.get('currentPage');
  }.property('currentPage', 'total'),
  isFirstPage: function(){
    return ! this.get('hasPreviousPage');
  }.property('currentPage', 'total'),
  isLastPage: function(){
    return ! this.get('hasNextPage');
  }.property('currentPage', 'total')
});

CBS.NotificationController = Ember.Controller.extend({
  responseText: null,
  actions: {
    answer: function(answer) {
      preloader();
      var notification = this.get('model');
      var self = this;

      if (Ember.$.inArray(answer, ['acknowledged', 'responding', 'unavailable']) < 0) {
        alert('Unknown answer '+answer);
        return false;
      }

      notification.answer(answer, function(response){
        removePreloader();
        if (response && response.error) {
          var errorview = CBS.ErrorModalView.create({
            title: 'Error',
            content: 'Cannot save notification response. Please try again later!'
          });

          errorview.append();
        }
        else {
          window.localStorage.removeItem('notifications');
          notification.set('response', answer);
          CBS.utilities.showSnackbar('Status set.');
        }
      });
    },
    sendResponse: function() {
      preloader();
      var notification = this.get('model');
      var self = this;

      notification.sendResponse(this.get('responseText'), function(response){
        removePreloader();
        if (response && response.error) {
          var errorview = CBS.ErrorModalView.create({
            title: 'Error',
            content: 'Cannot save notification response. Please try again later!'
          });

          errorview.append();
        }
        else {
          var currentResponses = notification.get('responses');
          currentResponses.push({
            message : self.get('responseText'),
            response_date : moment().format('YYYY-MM-DD HH:mm:ss')
          });
          notification.set('responses', currentResponses);
          notification.propertyWillChange('responses');
          notification.propertyDidChange('responses');
          self.set('responseText', null);
          CBS.utilities.showSnackbar('Response sent.');
        }
      });
    }
  }
});

CBS.ScheduleController = Ember.Controller.extend({
  schedule: []
});

CBS.AdminNotificationsController = Ember.Controller.extend({
  actions: {
    clear: function() {
      this.set('notifications', []);
    },
    refresh: function(callbackFunction) {
      var self = this;
      self.transitionToRoute('admin.notifications', 1);
    },
    prev: function(){
      this.transitionToRoute('admin.notifications', this.get('currentPage') - 1);
    },
    next: function(){
      this.transitionToRoute('admin.notifications', this.get('currentPage') + 1);
    }
  },
  needs: ['application'],
  notifications: [],
  currentUser: Ember.computed.alias('controllers.application.currentUser'),
  count: function(){
    return this.get('notifications').length;
  }.property('notifications'),
  hasPreviousPage: function(){
    return this.get('currentPage') > 1;
  }.property('currentPage', 'total'),
  hasNextPage: function(){
    return Math.ceil(this.get('total') / CBS.Notification.itemsPerPage) > this.get('currentPage');
  }.property('currentPage', 'total'),
  isFirstPage: function(){
    return ! this.get('hasPreviousPage');
  }.property('currentPage', 'total'),
  isLastPage: function(){
    return ! this.get('hasNextPage');
  }.property('currentPage', 'total')
});

CBS.AdminNotificationController = Ember.ObjectController.extend({
  refreshing: false,
  isRefreshing: function(){
    return this.get('refreshing');
  }.property('refreshing')
});

CBS.AdminAddnotificationController = Ember.Controller.extend({
  type: 'all',
  message: '',
  sendMail: false,
  actions: {
    activateType: function(name) {
      Ember.$('.notification-recipient-label:visible').fadeOut(100, function(){
        Ember.$('#label-'+name).fadeIn(100);
      });
      Ember.$('#send-to-select .btn').addClass('btn-default').removeClass('btn-primary');      
      Ember.$('#button-'+name).removeClass('btn-default').addClass('btn-primary').blur();
      if (Ember.$('.filter-details:visible').length) {
        Ember.$('.filter-details:visible').fadeOut(100, function(){
          Ember.$('#details-'+name).fadeIn(100);
        });
      }
      else {
        Ember.$('#details-'+name).fadeIn(100);
      }
      this.set('type', name);
    },
    selectItems: function(modelName, modelNamePlural) {
      var view = CBS.SelectModalView.create({
        modelName: modelName,
        modelNamePlural: modelNamePlural
      }); 
      CBS[modelName.capitalize()].all().then(function(items){
        view.set('items', items);
      });
      view.append();
    },
    send: function() {
      if (this.get('message').length < 2) {
        CBS.utilities.showSnackbar('Please enter a message.');
        $('#message').focus();
        return;
      }

      if (this.get('type') !== 'all' && ! $('.btn-xs:visible').length) {
        CBS.utilities.showSnackbar('Please choose at least one '+this.get('type')+'.');
        return;
      }

      var self = this;

      $('.send-notification').button('loading');

      $.post(
        api('/v1/notification'), 
        $('#add-notification-form').serialize()+'&message='+this.get('message')+'&type='+this.get('type'),
        function(response) {
          if (response && ! response.error) {
            self.transitionToRoute('admin.notification', response.notification.id);
            CBS.utilities.showSnackbar('Notification sent.');
            self.set('message', '');
          }
          else if (response && response.error) {
            CBS.utilities.showSnackbar('Error: '+response.message);
          }
          else {
            CBS.utilities.showSnackbar('An unknown error happened.'); 
          }

          $('.send-notification').button('reset');
        },
        'json'
      );
    }
  },
  remainingCharacters: function() {
    return 279 - this.get('message').length;
  }.property('message'),
  limitMessage: function() {
    var remaining = this.get('remainingCharacters');
    if (remaining < 0) {
      this.set('message', this.get('message').substring(0, 279));
    }
  }.property('remainingCharacters', 'message')
});

CBS.ModalView = Ember.View.extend({
    templateName: "modal",
    title: "",
    content: "",
    classNames: ["modal", "fade"],
    didInsertElement: function() {
        this.$().modal("show");
        this.$().one("hidden", this._viewDidHide);
    },
    _viewDidHide: function() {
        if ( ! this.isDestroyed) {
            this.destroy();
        }
    },
    actions: {
      close: function() {        
        this.$(".close").click();
      }
    }
});

CBS.ErrorModalView = CBS.ModalView.extend({
    templateName: "errormodal"
});

CBS.SuccessModalView = CBS.ModalView.extend({
    templateName: "successmodal"
});

CBS.NotificationModalView = CBS.ModalView.extend({
    templateName: "notificationmodal",
    cancelButtonLabel: "Dismiss",
    okButtonLabel: "Respond",
    okButtonAction: this.$(".close").click(), 
    actions: {
      ok: function() {
        this.send('okButtonAction');
      }
    }
});

CBS.SelectModalView = CBS.ModalView.extend({
    templateName: "select-modal",
    modelName: "",
    modelNamePlural: "",
    cancelButtonLabel: "Cancel",
    okButtonLabel: "Select",
    okButtonAction: function(){
      var selected = [];
      this.$().find('.fa-check-square').each(function(){
        var $item = $(this).closest('.list-group-item');
        selected.push({
          'id': $item.data('itemid'),
          'name': $item.find('.item-name').text()
        });
      });

      this.okCallback(selected);
    },
    okCallback: function(selected){
      var self = this;
      selected.forEach(function(e, i){
        var $button = $(
          '<span>'+
            '<button class="btn btn-xs btn-default">'+
              e.name+
            '</button>'+
            '<input type="hidden" name="'+self.get('modelNamePlural').toLowerCase()+'[]" value="'+e.id+'" />'+
          '</span>'
        );

        $('button', $button).click(function(event){
          event.preventDefault();
          $button.fadeOut(200, function(){
            $(this).remove();
          });
        });

        $('.'+self.get('modelName')+'.panel-body').append($button);
      });

      this.$().find('.close').click();
    },
    items: [],
    actions: {
      ok: function() {
        this.send('okButtonAction');
      }
    },
    didInsertElement: function() {
      this._super();
      var height = $(window).height() - 60;
      this.$().find('.modal-body').css({
        "height": height - 135,
        "overflow-y": "auto"
      });
      this.$().off('click', '.list-group-item').on('click', '.list-group-item', function(e){
        e.preventDefault();
        $(this).find('i.fa').toggleClass('fa-square-o').toggleClass('fa-check-square');
      });
    }
});

CBS.ObjectSelectorView = Ember.View.extend({
  templateName: "object-selector",
  modelName: "",
  modelNamePlural: "",
  getAddButtonId: function(){
    return "add-"+this.modelNamePlural;
  }
});

CBS.UserSelectorView = CBS.ObjectSelectorView.extend({
  modelName: "user",
  modelNamePlural: "users"
});
CBS.GroupSelectorView = CBS.ObjectSelectorView.extend({
  modelName: "group",
  modelNamePlural: "groups"
});
CBS.FilterSelectorView = CBS.ObjectSelectorView.extend({
  modelName: "filter",
  modelNamePlural: "filters"
});
CBS.TitleSelectorView = CBS.ObjectSelectorView.extend({
  modelName: "title",
  modelNamePlural: "titles"
});

CBS.CallbackResponseSuccessModalView = CBS.SuccessModalView.extend({
    actions: {
      close: function() {
        this.controller.controllerFor('callbacks').send('refresh');
        this._super();
        this.get('controller').transitionToRoute('callbacks');
      }
    }
});



CBS.TimeoffController = Ember.Controller.extend({
  needs: ['application']
});