/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

var app = {
    // Application Constructor
    initialize: function() {
        this.bindEvents();
    },
    // Bind Event Listeners
    //
    // Bind any events that are required on startup. Common events are:
    // 'load', 'deviceready', 'offline', and 'online'.
    bindEvents: function() {
        document.addEventListener('deviceready', this.onDeviceReady, false);
    },
    // deviceready Event Handler
    //
    // The scope of 'this' is the event. In order to call the 'receivedEvent'
    // function, we must explicity call 'app.receivedEvent(...);'
    onDeviceReady: function() {
        navigator.splashscreen.hide();
        app.receivedEvent('deviceready');
        if (window.StatusBar) {
          StatusBar.overlaysWebView(false);
          StatusBar.backgroundColorByHexString("#444444");
        }

        // Fix sticky headers with iOS keyboards
        $(document).on('focus', 'textarea,input,select', function() {
            $('.navbar.navbar-fixed-top').css('position', 'absolute');
        }).on('blur', 'textarea,input,select', function() {
            $('.navbar.navbar-fixed-top').css('position', '');
            setTimeout(function() {
                window.scrollTo(document.body.scrollLeft, document.body.scrollTop);
            }, 0);
        });
    },
    // Update DOM on a Received Event
    receivedEvent: function(id) {
    },

    successHandler: function(result) {
    },
    errorHandler:function(error) {
        alert(error);
    },
    tokenHandler: function(token) {
      Ember.$.post(
          api('/v1/apns-registration'),
          {
              token : token
          },
          function(){
          }
      );
    },
    onNotificationGCM: function(e) {
        switch( e.event )
        {
            case 'registered':
                if ( e.regid.length > 0 )
                {
                  Ember.$.post(
                      api('/v1/android-registration'),
                      {
                          registrationId : e.regid
                      },
                      function(){
                      }
                  );
                }
            break;
 
            case 'message':
              if (e.payload.callback_id) {
                  if (e.foreground) {
                      var modal = CBS.NotificationModalView.create({
                        title: 'New Callback',
                        content: 'You have a new callback opportunity: '+e.payload.title+', '+e.payload.message,
                        okButtonAction: function() {
                            this.$(".close").click();
                            app.goToCallback(e.payload.callback_id);
                        }
                      });

                      modal.append();
                  }
                  else {
                    app.goToCallback(e.payload.callback_id);
                  }
              }
              else if (e.payload.notification_id) {
                  if (e.foreground) {
                      var modal = CBS.NotificationModalView.create({
                        title: 'New Group Notification',
                        content: 'You have a new group notification: '+e.payload.message,
                        okButtonAction: function() {
                            this.$(".close").click();
                            app.goToNotification(e.payload.notification_id);
                        }
                      });

                      modal.append();
                  }
                  else {
                    app.goToNotification(e.payload.notification_id);
                  }
              }
            break;
 
            case 'error':
              alert('GCM error = '+e.msg);
            break;
 
            default:
              alert('An unknown GCM event has occurred');
              break;
        }
    },

    onNotificationAPN: function(e) {
        if (e.callback_id) {
            if (e.foreground) {
                var modal = CBS.NotificationModalView.create({
                  title: 'New Callback',
                  content: 'You have a new callback opportunity: '+e.title+', '+e.message,
                  okButtonAction: function() {
                      this.$(".close").click();
                      app.goToCallback(e.callback_id);
                  }
                });

                modal.append();
            }
            else {
              app.goToCallback(e.callback_id);
            }
        }
        else if (e.notification_id) {
            if (e.foreground) {
                var modal = CBS.NotificationModalView.create({
                  title: 'New Group Notification',
                  content: 'You have a new group notification: '+e.message,
                  okButtonAction: function() {
                      this.$(".close").click();
                      app.goToNotification(e.notification_id);
                  }
                });

                modal.append();
            }
            else {
              app.goToNotification(e.notification_id);
            }
        }
        
        if (e.badge) {
            var pushNotification = window.plugins.pushNotification;
            pushNotification.setApplicationIconBadgeNumber(app.successHandler, e.badge);
        }
    },


    goToCallback: function(callbackId) {
        preloader();
        var controller = CBS.__container__.lookup('controller:callbacks');
        var boundSend = controller.send.bind(controller);
        boundSend('refresh', function(callbacksController){
            removePreloader();
            callbacksController.transitionToRoute('callback', callbackId);
        });
    },
    goToNotification: function(notificationId) {
        preloader();
        var controller = CBS.__container__.lookup('controller:callbacks');
        controller.transitionToRoute('notification', notificationId);
    },

    registerPushNotificationServices: function() {
      if (typeof window.plugins !== 'undefined') {
        var pushNotification = window.plugins.pushNotification;
        if (device.platform == 'android' || device.platform == 'Android') {
          pushNotification.register(app.successHandler, app.errorHandler, {
              "senderID": "46792956770",
              "ecb": "app.onNotificationGCM"
          });
        }
        else {
          pushNotification.register(app.tokenHandler, app.errorHandler, {
              "badge": "true",
              "sound": "true",
              "alert": "true",
              "ecb":"app.onNotificationAPN"
          });
        }
      }
    }
};

function preloader() {
  $('#loading').fadeIn(200);
}
function removePreloader() {
  $('#loading').fadeOut(200);
}
