CBS.User = Ember.Object.extend({
	fullName: function() {
		return this.get('first_name')+' '+this.get('last_name');
	}.property('first_name', 'last_name'),
	isAdmin: function() {
		return this.get('role') == 'Administrator';
	}.property('role'),
	id: function() {
		return this.get('user_id');
	}.property('user_id'),
	name: function() {
		return this.get('fullName');
	}.property('first_name', 'last_name')
});

CBS.User.reopenClass({
	all: function(force) {
		var self = this;
		var usersInStorage = window.localStorage.getItem('users');
		if ( ! force && usersInStorage) {
			return new Ember.RSVP.Promise(function(resolve){
				resolve(self.itemsToObjects(JSON.parse(usersInStorage)));
			});
		}
		else {
			var self = this;
			return this.allFromAPI().then(function(data){
				var users = JSON.stringify(data);
				window.localStorage.setItem('users', users);
				return self.itemsToObjects(data);
			});
		}
	},
	allFromAPI: function() {
		return Ember.$.getJSON(api('/v1/users'));
	},
	itemsToObjects: function(users) {
		var userObjects = [];

		users.sortBy('last_name').forEach(function(item){
			userObjects.pushObject(CBS.User.create(item));
		});

		return userObjects;
	}
});

CBS.Group = Ember.Object.extend({
});

CBS.Group.reopenClass({
	all: function(force) {
		var self = this;
		var groupsInStorage = window.localStorage.getItem('groups');
		if ( ! force && groupsInStorage) {
			return new Ember.RSVP.Promise(function(resolve){
				resolve(self.itemsToObjects(JSON.parse(groupsInStorage)));
			});
		}
		else {
			var self = this;
			return this.allFromAPI().then(function(data){
				var groups = JSON.stringify(data);
				window.localStorage.setItem('groups', groups);
				return self.itemsToObjects(data);
			});
		}
	},
	allFromAPI: function() {
		return Ember.$.getJSON(api('/v1/groups'));
	},
	itemsToObjects: function(groups) {
		var groupObjects = [];

		groups.sortBy('name').forEach(function(item){
			groupObjects.pushObject(CBS.Group.create(item));
		});

		return groupObjects;
	}
});

CBS.Filter = Ember.Object.extend({
});

CBS.Filter.reopenClass({
	all: function(force) {
		var self = this;
		var filtersInStorage = window.localStorage.getItem('filters');
		if ( ! force && filtersInStorage) {
			return new Ember.RSVP.Promise(function(resolve){
				resolve(self.itemsToObjects(JSON.parse(filtersInStorage)));
			});
		}
		else {
			var self = this;
			return this.allFromAPI().then(function(data){
				var filters = JSON.stringify(data);
				window.localStorage.setItem('filters', filters);
				return self.itemsToObjects(data);
			});
		}
	},
	allFromAPI: function() {
		return Ember.$.getJSON(api('/v1/filters'));
	},
	itemsToObjects: function(filters) {
		var filterObjects = [];

		filters.sortBy('name').forEach(function(item){
			filterObjects.pushObject(CBS.Filter.create(item));
		});

		return filterObjects;
	}
});

CBS.Title = Ember.Object.extend({
});

CBS.Title.reopenClass({
	all: function(force) {
		var self = this;
		var titlesInStorage = window.localStorage.getItem('titles');
		if ( ! force && titlesInStorage) {
			return new Ember.RSVP.Promise(function(resolve){
				resolve(self.itemsToObjects(JSON.parse(titlesInStorage)));
			});
		}
		else {
			var self = this;
			return this.allFromAPI().then(function(data){
				var titles = JSON.stringify(data);
				window.localStorage.setItem('titles', titles);
				return self.itemsToObjects(data);
			});
		}
	},
	allFromAPI: function() {
		return Ember.$.getJSON(api('/v1/titles'));
	},
	itemsToObjects: function(titles) {
		var titleObjects = [];

		titles.sortBy('name').forEach(function(item){
			titleObjects.pushObject(CBS.Title.create(item));
		});

		return titleObjects;
	}
});

// CALLBACKS

CBS.Callback = Ember.Object.extend();

CBS.Callback.reopen({
	isSplittable: function() {
		return !! parseInt(this.get('splittable'));
	}.property('splittable'),
	hasWorkSites: function() {
		return !! parseInt(this.get('has_work_sites'));
	}.property('has_work_sites'),
	getWorkSites: function() {
		return this.get('work_sites');
	}.property('work_sites'),
	getParts: function() {
		return this.get('parts');
	}.property('parts'),
	deny: function(callback) {
		$.post(
			api('/v1/callback-record/'+this.get('callback_record_id')+'/deny'),
			{},
			callback,
			'json'
		);
	},
    accept: function(callback, part) {
      part = part || 'full';
      $.post(
        api('/v1/callback-record/'+this.get('callback_record_id')+'/accept'),
        {
          part : part
        },
        callback,
        'json'
      );
    },
    acceptWorkSite: function(callback, workSiteId) {
      $.post(
        api('/v1/callback-record/'+this.get('callback_record_id')+'/accept'),
        {
          workSiteId : workSiteId
        },
        callback,
        'json'
      );
    },
    getFriendlyWaitTime: function() {
    	var waitTime = this.get('response_time'),
    		hours,
    		minutes,
    		days;

    	if (waitTime == 0) {
    		return "no wait";
    	}
        else if (waitTime >= 60 && waitTime < 24 * 60) {
            hours = Math.floor(waitTime /  60);
            minutes = waitTime % 60;

            return Math.round(hours)+" hours"+(minutes > 0 ? " "+minutes+" minutes" : '');
        }
        else if (waitTime >= 24 * 60) {
            days = Math.floor(waitTime / (24 * 60));
            hours = Math.floor((waitTime % (24 * 60)) / 60);
            return Math.round(days)+" days"+(hours > 0 ? " "+hours+" hours" : '');
        }
        else {
            return waitTime+" minutes";
        }
    }.property('response_time')
});

CBS.Callback.reopenClass({
	all: function(force) {
		var self = this;
		return this.allFromAPI().done(removePreloader).then(function(data){
			var callbacks = JSON.stringify(data);
			return self.itemsToObjects(data);
		});
	},
	allFromAPI: function() {
		return Ember.$.getJSON(api('/v1/me/active-callbacks'));
	},
	itemsToObjects: function(callbacks) {
		var callbackObjects = [];

		callbacks.forEach(function(item){
			callbackObjects.pushObject(CBS.Callback.create(item));
		});

		return callbackObjects;
	}
});



// NOTIFICATIONS 

CBS.UserNotification = Ember.Object.extend();

CBS.UserNotification.reopen({
	answer: function(answer, callback) {
		$.post(
			api('/v1/me/notification/'+this.get('notification_id')+'/answer'),
			{
				answer: answer
			},
			callback,
			'json'
		);
	},
	sendResponse: function(response, callback) {
		$.post(
			api('/v1/me/notification/'+this.get('notification_id')+'/response'),
			{
				response: response
			},
			callback,
			'json'
		);
	},
	excerpt: function(){
		var message = this.get('message');
		if (message.length > 50) {
			message = message.substring(0, 50) + '...'
		}

		return message;
	}.property('message'),
	isAnswered: function(){
		if (this.get('response') && this.get('response').length) {
			return true;
		}

		return false;
	}.property('response'),
	isAcknowledged: function(){
		return (this.get('response') && this.get('response') == 'acknowledged');
	}.property('response'),
	isResponding: function(){
		return (this.get('response') && this.get('response') == 'responding');
	}.property('response'),
	isUnavailable: function(){
		return (this.get('response') && this.get('response') == 'unavailable');
	}.property('response'),
	displayDate: function(){
		var displayDate = moment(this.get('timestamp'));
		return displayDate.calendar();
	}.property('timestamp'),
	getResponses: function(){
		return this.get('responses').sortBy('response_date').reverse();
	}.property('responses')
});

CBS.UserNotification.reopenClass({
	itemsPerPage: 10,
	all: function(page, force) {
		page = parseInt(page || 1);
		var offset = (page - 1) * this.itemsPerPage;

		var notificationsInStorage = window.localStorage.getItem('user-notifications');
		if ( ! force && notificationsInStorage) {
			removePreloader();
			notificationsInStorage = JSON.parse(notificationsInStorage);
			return this.itemsToObjects(notificationsInStorage.splice(offset, this.itemsPerPage));
		}
		else {
			var self = this;
			return this.allFromAPI(offset, this.itemsPerPage).then(function(data){
				var notificationsData = JSON.stringify(data.notifications);
				var notificationsTotal = JSON.stringify(data.total);

				window.localStorage.setItem('user-notifications', notificationsData);
				window.localStorage.setItem('user-notificationsTotal', notificationsTotal);

				removePreloader();
				return self.itemsToObjects(data.notifications);
			});
		}
	},
	allFromAPI: function(offset, itemsPerPage) {
		return Ember.$.post(api('/v1/me/notifications'), {
			offset: offset,
			limit: itemsPerPage
		}, function(){}, 'json');
	},
	itemsToObjects: function(notifications) {
		var notificationObjects = [];

		notifications.forEach(function(item){
			notificationObjects.pushObject(CBS.UserNotification.create(item));
		});

		return notificationObjects;
	},
	findForUser: function(notificationId) {
		return Ember.$.getJSON(api('/v1/me/notification/'+notificationId))
			.then(function(json){
				var notification = CBS.UserNotification.create(json);
				return notification;
			});
	}
});

CBS.Notification = Ember.Object.extend();

CBS.Notification.reopen({
	excerpt: function(){
		var message = this.get('message');
		if (message.length > 50) {
			message = message.substring(0, 50) + '...'
		}

		return message;
	}.property('message'),
	displayDate: function(){
		var displayDate = moment(this.get('timestamp'));
		var now = moment();

		if (displayDate.isSame(now, 'day')) {
			return displayDate.format('h:mma');
		}
		else if (displayDate.isSame(now, 'year')) {
			return displayDate.format('D MMM');
		}
		else {
			return displayDate.format('M/D/YY');
		}
	}.property('timestamp'),
	fullDisplayDate: function(){
		var displayDate = moment(this.get('timestamp'));
		return displayDate.format('M/D/YY h:mma');
	}.property('timestamp'),
	total: function(){
		return parseInt(this.get('success'))+parseInt(this.get('failure'));
	}.property('success', 'failure'),
	successRate: function(){
		return parseInt(this.get('success')) / parseInt(this.get('total'));
	},
	sent: function(){
		return this.successRate() >= 0.75;
	}.property('success', 'failure'),
	partiallySent: function(){
		return (this.successRate() > 0.25 && this.successRate() < 0.75);
	}.property('success', 'failure'),
	notSent: function(){
		return this.successRate() <= 0.25;
	}.property('success', 'failure'),
	getResponses: function(){
		return this.get('responses').sortBy('response_date').reverse();
	}.property('responses'),
	getUsers: function(){
		return this.get('users');
	}.property('users'),
	getSentTo: function(){
		var sentTo = this.get('sent_to');
		if (sentTo.length > 20) {
			return this.get('total')+' recipient'+(this.get('total') <= 1 ? '' : 's');
		}

		return sentTo;
	}.property('sent_to'),
	isLive: function(){
		var startDate = moment(this.get('timestamp'));
		var anHourAgo = moment().subtract(1, 'hours');

		return startDate.isAfter(anHourAgo);
	}.property('timestamp')
});

CBS.Notification.reopenClass({
	itemsPerPage: 10,
	all: function(page, force) {
		page = parseInt(page || 1);
		var offset = (page - 1) * this.itemsPerPage;

		var notificationsInStorage = window.localStorage.getItem('notifications');
		if ( ! force && notificationsInStorage) {
			removePreloader();
			notificationsInStorage = JSON.parse(notificationsInStorage);
			return this.itemsToObjects(notificationsInStorage.splice(offset, this.itemsPerPage));
		}
		else {
			var self = this;
			return this.allFromAPI(offset, this.itemsPerPage).then(function(data){
				var notificationsData = JSON.stringify(data.notifications);
				var notificationsTotal = JSON.stringify(data.total);

				window.localStorage.setItem('notifications', notificationsData);
				window.localStorage.setItem('notificationsTotal', notificationsTotal);

				removePreloader();
				return self.itemsToObjects(data.notifications);
			});
		}
	},
	allFromAPI: function(offset, itemsPerPage) {
		return Ember.$.get(api('/v1/notification'), {
			offset: offset,
			limit: itemsPerPage
		}, function(){}, 'json');
	},
	itemsToObjects: function(notifications) {
		var notificationObjects = [];

		notifications.forEach(function(item){
			notificationObjects.pushObject(CBS.Notification.create(item));
		});

		return notificationObjects;
	},
	find: function(notificationId) {
		return Ember.$.getJSON(api('/v1/notification/'+notificationId))
			.then(function(json){
				json.users = json.users.map(function(elem){
					return CBS.NotificationUser.create(elem);
				});
				var notification = CBS.Notification.create(json);
				return notification;
			});
	}
});

CBS.NotificationUser = Ember.Object.extend({
	statusClass: function(){
		switch (this.get('status')) {
			case 'failed':
			case 'busy':
			case 'undelivered':
				return 'fa-exclamation text-danger';
				break;

			case 'sent':
			case 'completed':
			case 'delivered':
				return 'fa-check text-success';
				break;

			case 'queued':
			case 'no-answer':
			case 'in-progress':
			default:
				return 'fa-spinner fa-spin';
				break;
		}
	}.property('status'),
	methodClass: function(){
		switch (this.get('contact_method')) {
			case 'sms':
				return 'fa-comment';
				break;

			case 'call':
				return 'fa-phone';
				break;

			case 'app':
				return 'fa-mobile';
				break;

			case 'pager':
				return 'fa-rss';
				break;

			default:
				return 'fa-question text-warning';
		}
	}.property('contact_method'),
	responseClass: function(){
		switch (this.get('response')) {
			case 'unavailable':
				return 'fa-remove text-danger';
				break;

			case 'acknowledged':
				return 'fa-check-square text-warning';
				break;

			case 'responding':
				return 'fa-check text-success';
				break;

			default:
				return '';
		}
	}.property('response'),
	notProcessed: function(){
		return ! this.get('processed');
	}.property('processed')
});

CBS.Schedule = Ember.Object.extend()

CBS.Schedule.reopen({
	getDisplayStart: function() {
		return moment(this.get('start')).format('M/DD h:mma');
	}.property('start'),
	getDisplayEnd: function() {
		return moment(this.get('end')).format('M/DD h:mma');
	}.property('end')
});

CBS.Schedule.reopenClass({
	comingUp: function(force) {
		var self = this;
		var scheduleInStorage = window.localStorage.getItem('user-schedule');
		if ( ! force && scheduleInStorage) {
			return new Ember.RSVP.Promise(function(resolve){
				resolve(self.itemsToObjects(JSON.parse(scheduleInStorage)));
			});
		}
		else {
			return Ember.$.getJSON(api('/v1/me/schedule')).then(function(data){
				var schedule = JSON.stringify(data);
				window.localStorage.setItem('user-schedule', schedule);
				return self.itemsToObjects(data);
			}, function() {
				CBS.utilities.showSnackbar("Cannot load schedule");
			})
			.always(function(){
				removePreloader();
			});
		}
	},
	itemsToObjects: function(schedule) {
		var scheduleObjects = [];

		schedule.forEach(function(item){
			scheduleObjects.pushObject(CBS.Schedule.create(item));
		});

		return scheduleObjects;
	}
});