CBS.Router.map(function() {
  this.route('login');
  this.resource('callbacks');
  this.resource('callback', { path: "/callback/:callback_id" });
  this.resource('notifications', { path: "/notifications/page/:page" });
  this.resource('notification', { path: "/notification/:notification_id" });

  this.resource('schedule', { path: "/schedule/page/:page" });

  this.resource('timeoff', { path: "/timeoff/list/:page" }, function(){
  	this.route('request');
  });

  this.resource('admin', function() {
  	this.route('notifications', { path: "/notifications/page/:page" });
    this.route('notification', { path: "/notification/:notification_id" });
    this.route('addnotification');
  });
});

CBS.ResetScroll = Ember.Mixin.create({
  activate: function() {
    this._super();
    window.scrollTo(0,0);
  }
});

CBS.LoginRoute = Ember.Route.extend({
	setupController: function(controller, model) {
		this._super(controller, model);
		removePreloader();
	},
	activate: function() {
    	this._super();
		if (window.StatusBar) {
			StatusBar.overlaysWebView(true);
			StatusBar.styleDefault();
		}
	},
	deactivate: function() {
    	this._super();
		if (window.StatusBar) {
			StatusBar.overlaysWebView(false);
			StatusBar.styleBlackOpaque();
			StatusBar.backgroundColorByHexString("#444444");
		}
	}
});

CBS.IndexRoute = Ember.Route.extend(Ember.SimpleAuth.AuthenticatedRouteMixin, CBS.ResetScroll, {
	model: function() {
		var self = this;
		var loadedUser = window.localStorage.getItem('current-user');
		if (loadedUser) {
			var currentUser = CBS.User.create(JSON.parse(loadedUser));
			this.controllerFor('application').set('currentUser', currentUser);
			return currentUser;
		}
		
		return Ember.$.getJSON(api('/v1/me')).then(function(data){
			window.localStorage.setItem('current-user', JSON.stringify(data));
			var currentUser = CBS.User.create(data);
			self.controllerFor('application').set('currentUser', currentUser);
			return currentUser;
		});
	},
	setupController: function(controller, model) {
		this.controllerFor('application')
			.set('mainPage', true)
			.set('pageTitle', 'Dashboard')
			.set('userName', model.get('fullName'));
		controller.set('model', model);

		CBS.Schedule.comingUp().then(function(schedule){
			if (typeof schedule[0] === 'undefined' ||
				moment(schedule[0].end).isBefore(moment())) {
				CBS.Schedule.comingUp(true).then(function(schedule){
					controller.set('schedule', schedule);
				});
			}
			else {
				controller.set('schedule', schedule);
			}
		});
	}	
});

CBS.CallbacksRoute = Ember.Route.extend(Ember.SimpleAuth.AuthenticatedRouteMixin, CBS.ResetScroll, {
	model: function() {
		return CBS.Callback.all();
	},
	setupController: function(controller, callbacks) {
		controller.set('callbacks', callbacks);
		this.controllerFor('application')
			.set('mainPage', false)
			.set('pageTitle', 'Callbacks');
	},
	renderTemplate: function() {
		this._super();
		this.render('refresh', {
			into: 'application',
			outlet: 'actionButtons',
			controller: 'callbacks'
		});
	},
	actions: {
	    backButton: function(){
	      this.transitionTo('index');
	    }
	}
});
CBS.CallbacksView = Ember.View.extend(CBS.SlideInView);

CBS.CallbackRoute = Ember.Route.extend(Ember.SimpleAuth.AuthenticatedRouteMixin, CBS.ResetScroll, {
	model: function(params) {
		return CBS.Callback.all().then(function(callbacks){
			var callback = callbacks.filterBy('callback_id', params.callback_id).get('firstObject');
			return callback;
		});
	},
	setupController: function(controller, callback) {
		this._super(controller, callback);
		this.controllerFor('application')
			.set('mainPage', false)
			.set('pageTitle', 'Respond');
	},
	actions: {
	    loading: function() {
	      preloader();
	    },
	    backButton: function(){
	      this.transitionTo('callbacks');
	    }
	}
});



CBS.NotificationsRoute = Ember.Route.extend(Ember.SimpleAuth.AuthenticatedRouteMixin, CBS.ResetScroll, {
	model: function(params) {
		this.controllerFor('notifications').set('currentPage', parseInt(params.page || 1));
		return CBS.UserNotification.all(params.page, true);
	},
	setupController: function(controller, notifications) {
		controller.set('notifications', notifications);
		controller.set('total', JSON.parse(window.localStorage.getItem('notificationsTotal')));
		this.controllerFor('application')
			.set('mainPage', false)
			.set('pageTitle', 'Notifications');
		controller.set('currentUser', this.controllerFor('application').get('currentUser'));
      	window.scrollTo(0, 0);
	},
	renderTemplate: function() {
		this._super();
		this.render('actions-notifications', {
			into: 'application',
			outlet: 'actionButtons',
			controller: 'notifications'
		});
	},
	actions: {
	    loading: function() {
	      preloader();
	    },
	    backButton: function(){
	      this.transitionTo('index');
	    }
	}
});

CBS.NotificationRoute = Ember.Route.extend(Ember.SimpleAuth.AuthenticatedRouteMixin, CBS.ResetScroll, {
	model: function(params) {
		return CBS.UserNotification.findForUser(params.notification_id)
			.done(function(){
				removePreloader();
			});
	},
	setupController: function(controller, callback) {
		this._super(controller, callback);
		this.controllerFor('application')
			.set('mainPage', false)
			.set('pageTitle', 'Respond');
	},
	actions: {
	    loading: function() {
	      preloader();
	    },
	    backButton: function(){
	      var lastPage = this.controllerFor('notifications').get('currentPage');
	      this.transitionTo('notifications', lastPage);
	    }
	}
});

CBS.ScheduleRoute = Ember.Route.extend(Ember.SimpleAuth.AuthenticatedRouteMixin, CBS.ResetScroll, {
	model: function(params) {
		return CBS.Schedule.comingUp(true); // Always load the latest data
	},
	setupController: function(controller, schedule) {
		controller.set('schedule', schedule);
		this.controllerFor('application')
			.set('mainPage', false)
			.set('pageTitle', 'My Schedule');
	},
	actions: {
	    loading: function() {
	      preloader();
	    },
	    backButton: function(){
	      this.transitionTo('index');
	    }
	},
	activate: function() {
		this._super();
	    $('body').addClass('schedule-page');
	},
	deactivate: function(){
    	this._super();
		$('body').removeClass('schedule-page');
	}
});
CBS.ScheduleView = Ember.View.extend(CBS.SlideInView);

CBS.TimeoffRoute = Ember.Route.extend(Ember.SimpleAuth.AuthenticatedRouteMixin, CBS.ResetScroll, {
	setupController: function(controller, schedule) {
		controller.set('schedule', schedule);
		this.controllerFor('application')
			.set('mainPage', false)
			.set('pageTitle', 'Time Off');
	},
	actions: {
	    loading: function() {
	      preloader();
	    },
	    backButton: function(){
	      this.transitionTo('index');
	    }
	},
	renderTemplate: function() {
		this._super();
		this.render('actions-timeoff', {
			into: 'application',
			outlet: 'actionButtons',
			controller: 'timeoff'
		});
	},
	activate: function() {
		this._super();
	    $('body').addClass('timeoff-page');
	},
	deactivate: function(){
    	this._super();
		$('body').removeClass('timeoff-page');
	}
});

CBS.AdminRouteMixin = Ember.Mixin.create({
	activate: function() {
    	this._super();
		$('body').addClass('admin-page');
		if (window.StatusBar) {
			StatusBar.backgroundColorByHexString("#000000");
		}
	},
	deactivate: function() {
    	this._super();
		$('body').removeClass('admin-page');
		if (window.StatusBar) {
			StatusBar.backgroundColorByHexString("#444444");
		}
	}
});

CBS.AdminNotificationsRoute = Ember.Route.extend(Ember.SimpleAuth.AuthenticatedRouteMixin, CBS.ResetScroll, CBS.AdminRouteMixin, {
	model: function(params) {
		this.controllerFor('admin.notifications').set('currentPage', parseInt(params.page || 1));
		return CBS.Notification.all(params.page, true);
	},
	setupController: function(controller, notifications) {
		controller.set('notifications', notifications);
		controller.set('total', JSON.parse(window.localStorage.getItem('notificationsTotal')));
		this.controllerFor('application')
			.set('mainPage', false)
			.set('pageTitle', 'Notifications');
	},
	renderTemplate: function() {
		this._super();
		this.render('actions-notifications', {
			into: 'application',
			outlet: 'actionButtons',
			controller: 'admin-notifications'
		});
	},
	actions: {
	    loading: function() {
	      preloader();
	    },
	    backButton: function(){
	      this.transitionTo('index');
	    }
	}
});

CBS.AdminAddnotificationRoute = Ember.Route.extend(Ember.SimpleAuth.AuthenticatedRouteMixin, CBS.ResetScroll, CBS.AdminRouteMixin, {
	setupController: function(controller, model) {
		this.controllerFor('application')
			.set('mainPage', false)
			.set('pageTitle', 'New Notification');
	},
	actions: {
	    loading: function() {
	      preloader();
	    },
	    backButton: function(){
	      this.transitionTo('admin.notifications');
	    }
	}
});

CBS.AdminNotificationRoute = Ember.Route.extend(Ember.SimpleAuth.AuthenticatedRouteMixin, CBS.ResetScroll, CBS.AdminRouteMixin, {
	model: function(params) {
		return CBS.Notification.find(params.notification_id)
			.done(function(){
				removePreloader();
			});
	},
	setupController: function(controller, model) {
		this._super(controller, model);
		this.controllerFor('application')
			.set('mainPage', false)
			.set('pageTitle', 'Notification Details');

		this.controller.set('model', model);
		if (model.get('isLive')) {
			this.startRefreshing();
		}
	},
	startRefreshing: function(){
		this.controller.set('refreshing', true);
		Em.run.later(this, this.refresh, 2000);
	},
	refresh: function(){
		var model = this.controller.get('model');
		if ( ! this.controller.get('refreshing') || ! model.get('isLive'))
	  		return;

	  	var self = this;

	  	this.model({ notification_id: model.get('notification_id') })
	  		.then(function(notification){
	  			model.set('users', notification.users);
	  			Em.run.later(self, self.refresh, 2000);
	  		});
	},
	actions: {
	    loading: function() {
	      if (this.controller && ! this.controller.get('refreshing'))
	      	preloader();
	    },
	    backButton: function(){
	      var lastPage = this.controller.get('currentPage');
	      this.transitionTo('admin.notifications', lastPage);
	    },
	    willTransition: function(transition){
	      if (transition.targetName !== 'admin.notification') {
	      	this.controller.set('refreshing', false);
	      }
	    }
	}
});